import requests
import json

from deepwto.constants import available_ds, available_article, cited_by_ds


class AppSyncClient:
    latest_version = '1.0.0'
    available_ds_num = len(available_ds)
    available_ds = available_ds

    available_article_num = len(available_article)
    available_article = available_article

    def __init__(self, api_key, endpoint_url):
        self.api_key = api_key
        self.endpoint_url = endpoint_url
        self.headers = {
                'Content-Type': "application/graphql",
                'x-api-key': api_key,
                'cache-control': "no-cache"
        }

    def execute_gql(self, query):
        payload_obj = {"query": query}
        payload = json.dumps(payload_obj)
        response = requests.request("POST",
                                    self.endpoint_url,
                                    data=payload,
                                    headers=self.headers)
        return response

    def get_factual(self, ds: int, version: str = "1.0.0"):
        assert ds in self.available_ds, "Make sure choose ds number from " \
                                        "available_ds"
        ds = "{}".format(str(ds))
        version = "\"{}\"".format(version)

        query = """
                query GetFactual{{
                    getFactual(
                        ds: {0}, 
                        version: {1}) {{
                               factual
                            }}
                        }}
                """.format(ds, version)

        res = self.execute_gql(query).json()
        return res['data']['getFactual']['factual']

    def get_article(self, article: str, version: str = "1.0.0"):
        assert article in self.available_article, "Make sure choose article " \
                                                  "from available_article "

        article = "\"{}\"".format(article)
        version = "\"{}\"".format(version)
        query = """
                query GetGATT{{
                    getGATT(
                        article: {0}, 
                        version: {1}) {{
                               content
                            }}
                        }}
                """.format(article, version)

        res = self.execute_gql(query).json()
        return res['data']['getGATT']['content']

    def get_label(self, ds:int, article: str, version: str = "1.0.0"):
        assert ds in self.available_ds, "Make sure choose ds number from " \
                                        "available_ds"
        assert article in self.available_article, "Make sure choose article " \
                                                  "from available_article "

        ds_art = "\"{}\"".format(str(ds)+"_"+article)
        version = "\"{}\"".format(version)
        query = """
                query GetLabel{{
                    getLabel(
                        ds_art: {0}, 
                        version: {1}) {{
                               cited
                            }}
                        }}
                """.format(ds_art, version)

        res = self.execute_gql(query).json()
        return res['data']['getLabel']['cited']

    @staticmethod
    def get_cited_by_ds(self, ds: int):
        return cited_by_ds[ds]
